/*

This source file is part of the Solar Visitor program, (c) 2007
Graeme Sheppard. Released under the GPLv2.

*/


#include "Ogre.h"
#include "OgreKeyEvent.h"
#include "OgreEventListeners.h"

#include "Listener.h"
#include "Types.h"
#include "Platform.h"

using namespace Ogre;


Listener::Listener (RenderWindow *oWin, Camera *oCam, SceneManager *oSMgr,
  int iCode)
{
  Overlay *oSplashOverlay;
  Light *light;
  unsigned int iCount;
  
  oRenderWindow = oWin;
  oCamera = oCam;
  oSceneMgr = oSMgr;
  iModeCode = iCode;
  iReturnCode = sv_exit_normal;

  bContinue = true;
  bShiftPressed = false;
  iSolarVisitorTime = 0;
  iSolarTimeOrigin = iSolarVisitorTime;
  iSolarVisitorTimeLast = !iSolarVisitorTime;
  iTimeWarp= 200;
  iUserTimeOrigin = getUserTime();
  bMobileLight = false;
  rRotate = 72;
  rMove = 200;
  oDirection = Vector3::ZERO;
  iTimeKeysReleased = 0;
  iTimeLastDebugMsg = 0;
  iNumUserLights = 0;
  bStatsOn = false;
  bSimpleFPS = false;
  sScreenShotFileName = "";
  iScreenShotCount = 0;

  oCameraNode = oSceneMgr->getRootSceneNode()->createChildSceneNode();
  oCameraYawNode = oCameraNode->createChildSceneNode();
  oCameraPitchNode = oCameraYawNode->createChildSceneNode();
  oCameraRollNode = oCameraPitchNode->createChildSceneNode();
  resetView();
  oCameraRollNode->attachObject (oCamera);

  light = oSceneMgr->createLight ("Light1");
  light->setType (Light::LT_POINT);
  light->setPosition (Vector3 (0, 0, 0));
  light->setDiffuseColour (ColourValue::White);
  light->setSpecularColour (ColourValue::Black);

  oSplashOverlay = OverlayManager::getSingleton().
    getByName ("Solar/SplashOverlay");
  oSplashOverlay->show();
  oRenderWindow->update (true);

  oEventProcessor = new EventProcessor();
  oEventProcessor->initialise (oRenderWindow);
  oEventProcessor->startProcessingEvents();
  oEventProcessor->addKeyListener (this);
  oEventProcessor->addMouseListener (this);
  oEventProcessor->addMouseMotionListener (this);

  oOgreOverlay = OverlayManager::getSingleton().getByName ("Core/DebugOverlay");

  oSolarOverlay = OverlayManager::getSingleton().
    getByName ("Solar/SolarOverlay");
  oSolarOverlay->show();
  sTabMenu = "";

  oSolarSystem = new SolarSystem (oSceneMgr);
  oSolarSystem->addPlanets();

  if (iModeCode == sv_start_earth_pic)
  {
    iTimeWarp = 0;
    iSolarVisitorTimeLast = iSolarVisitorTime;
    for (iCount = 0; iCount < oSolarSystem->oPlanets.size(); iCount++)
      plotPlanet (oSolarSystem->oPlanets [iCount], 2600);
    plotPlanet (oSolarSystem->findPlanet ("Earth"), -300);
    plotPlanet (oSolarSystem->findPlanet ("Moon"), 80);
  }

//  causes segfault on exit, use hide()
//  OverlayManager::getSingleton().destroy (oSplashOverlay);
  oSplashOverlay->hide();
  oSceneMgr->setSkyBox (true, "Solar/SolarStars", 100000);
}


Listener::~Listener()
{
  delete oEventProcessor;
}


bool Listener::frameStarted (const FrameEvent &evt)
{
  static int iIntroTime = 0;
  unsigned int iCount;

  int iTime = getUserTime();
  
  if (oDirection == Vector3::ZERO)
    iTimeKeysReleased = iTime;
  else
  {
    oCameraNode->translate (oCameraYawNode->getOrientation() * 
      oCameraPitchNode->getOrientation() *
      oCameraRollNode->getOrientation() *
      oDirection * evt.timeSinceLastFrame * 
      pow (((iTime - iTimeKeysReleased) / 1000.0), 2), SceneNode::TS_LOCAL);
  }

  if (iTimeWarp != 0)
  {
    iSolarVisitorTime = iSolarTimeOrigin +
      (sv64 (iTime - iUserTimeOrigin) * iTimeWarp) / sv64 (1000);
  }

  if (iSolarVisitorTime != iSolarVisitorTimeLast)
  {
    iSolarVisitorTimeLast = iSolarVisitorTime;
    for (iCount = 0; iCount < oSolarSystem->oPlanets.size(); iCount++)
      plotPlanet (oSolarSystem->oPlanets [iCount], iSolarVisitorTime);
  }

  if (iModeCode == 0) // ie, not sv_start_plain
  {
    if (iIntroTime == 0)
      iIntroTime = iTime;
    else
    if (iIntroTime != -1 &&
      iTime- iIntroTime > 1500 && iTime - iIntroTime < 2000)
    {
      iIntroTime = -1;
      showDebugText ("Press F1 for help");
    }
  }

  return bContinue;
}


bool Listener::frameEnded (const FrameEvent &evt)
{
  int iTime = getUserTime();

  if (iTime != 0 && iTime > iTimeLastDebugMsg + 2000)
    showDebugText (NULL);

  if (sScreenShotFileName != "")
  {
    oRenderWindow->writeContentsToFile (sScreenShotFileName);
    showDebugText (String ("Wrote " + sScreenShotFileName).c_str());
    showTabMenu ("Hud");
    sScreenShotFileName = "";
  }
  updateStats();
  return true;
}


void Listener::mouseMoved (MouseEvent *evt)
{
  if (!bShiftPressed)
  {
    oCameraYawNode->yaw (Degree (-evt->getRelX() * rRotate));
    oCameraPitchNode->pitch (Degree (-evt->getRelY() * rRotate));
  }
  else
    oCameraRollNode->roll (Degree (+evt->getRelX() * rRotate));
}


void Listener::moveKey (int iKey, Real rDisplacement, bool bShift)
{
  switch (iKey)
  {
    case KC_LSHIFT:
    case KC_RSHIFT:
      bShiftPressed = bShift;
      break;

    case KC_UP:
    case KC_W:
      oDirection.z = -rDisplacement;
      break;

    case KC_DOWN:
    case KC_S:
      oDirection.z = +rDisplacement;
      break;
 
    case KC_LEFT:
    case KC_A:
      oDirection.x = -rDisplacement;
      break;
 
    case KC_RIGHT:
    case KC_D:
      oDirection.x = +rDisplacement;
      break;
 
    case KC_PGUP:
    case KC_Q:
      oDirection.y = +rDisplacement;
      break;

    case KC_PGDOWN:
    case KC_Z:
      oDirection.y = -rDisplacement;
      break;
  }
}

    
void Listener::keyPressed (KeyEvent *evt)
{
  static const int iNumKeys[] =
  {KC_0, KC_1, KC_2, KC_3, KC_4, KC_5, KC_6, KC_7, KC_8, KC_9, -1};
  int iKey, iIndex, iNumKey, iTemp;

  iKey = evt->getKey();
  iIndex = 0;
  iNumKey = -1;

  while (true)
  {
    iTemp = iNumKeys [iIndex];
    if (iTemp < 0)
      break;
    if (iKey == iTemp)
    {
      iNumKey = iIndex;
      break;
    }
    iIndex++;
  }

  if (iNumKey >= 0)
    numPress (iNumKey);
  else switch (iKey)
  {
    case KC_ESCAPE:
      bContinue = false;
      break;
    
    case KC_F1:
      static const char *szHelp = "Solar Vistor v0.2.0\n\n"
        "mouse: look\n"
        "arrows: move\n"
        "F1: show/hide help\n"
        "Tab: show/hide menu\n"
        "ESC: quit";
      setCommandText (cmpCommandText (szHelp) ? "" : szHelp);
      break;

    case KC_TAB:
      if (sTabMenu == "")
        showTabMenu ("0");
      else
        setCommandText (NULL);
      break;

    default:
      moveKey (iKey, rMove, true);
  }
}


void Listener::keyReleased (KeyEvent *evt)
{
  moveKey (evt->getKey(), 0, false);
}

