/*

This source file is part of the Solar Visitor program, (c) 2007
Graeme Sheppard. Released under the GPLv2.

*/


#include "Ogre.h"

#include "Listener.h"
#include "Types.h"
#include "Platform.h"

using namespace Ogre;


void Listener::setCommandText (const char *szText, const char *szTabName)
{
  sCommandText = szText == NULL ? "" : szText;
  sTabMenu = szTabName == NULL ? "" : szTabName;
  OverlayManager::getSingleton().getOverlayElement ("Solar/TextOut")->
    setCaption (sCommandText.c_str());
}


bool Listener::cmpCommandText (const char *text)
{
  return (strcmp (text, sCommandText.c_str()) == 0) ? true : false;
}


void Listener::showDebugText (const char *szText)
{
  OverlayManager::getSingleton().getOverlayElement ("Solar/DebugOut")->
    setCaption (szText == NULL ? "" : szText);
  iTimeLastDebugMsg = szText == NULL ? 0 : getUserTime();
}


void Listener::showTabMenu (const char *szMenuName)
{
  String sText, sLine;
  char cLetter;
  int iCount;

  iCount = 0;
  sLine = "";
  cLetter = '1';
        
  for (;;)
  {
    sText = oMenu.getMenuItem (szMenuName, iCount);
    if (sText == "" || cLetter > '9')
      break;
    if (sText [0] == '_')
      sText.erase (0,1);
    iCount++;
    sLine += cLetter++;
    sLine += ":" + sText + " ";
  }
  
  setCommandText (sLine.c_str(), szMenuName);
}


bool Listener::cmpCommand (const char *szAction)
{
  return strcmp (szAction, szTextCommand) == 0 ? true : false;
}


void Listener::resetView()
{
  oCameraNode->setPosition (3500.0, 0.0, -8000.0);
  oCameraYawNode->resetOrientation();
  oCameraPitchNode->resetOrientation();
  oCameraRollNode->resetOrientation();
  oCameraYawNode->yaw (Degree (-180.0));
}


void Listener::plotPlanet (Planet *oPlanet, sv64 iTime)
{
  SceneNode *oNode;

  oSolarSystem->updatePosition (oPlanet, iTime);
  oNode = oSceneMgr->getSceneNode (oPlanet->sName.c_str());
  oNode->setPosition (oPlanet->rX, oPlanet->rY, oPlanet->rZ);
  oNode->setOrientation (oPlanet->oOrientationNow);
}


void Listener::numPress (int iKey)
{
  Quaternion oQuat;
  Light *oUserLight;
  String sText;
  int iCount;

  if (sTabMenu == "")
    return;

  sText = oMenu.getMenuItem (sTabMenu.c_str(), iKey - 1);
  if (sText == "")
    return;

  if (sText [0] == '_')
  {
    sText.erase (0,1);
    showTabMenu (sText.c_str());
    return;
  }

  szTextCommand = sText.c_str();
  if (cmpCommand ("New Fixed Light"))
  {
    if (iNumUserLights >= iMaxUserLights)
      showDebugText (String ("At maximum " + toString (iMaxUserLights) +
        " lights").c_str());
    else
    {
      sText = "UserLight" + toString (iNumUserLights++);
      oUserLight = oSceneMgr->createLight (sText.c_str());
      oUserLight->setType (Light::LT_POINT);
      oUserLight->setDiffuseColour (ColourValue::White);
      oUserLight->setSpecularColour (ColourValue::Black);
      oUserLight->setPosition (oCameraNode->getPosition());
    }
  }
  else
  if (cmpCommand ("Remove User Lights"))
  {
    for (iCount = 0; iCount < iNumUserLights; iCount++)
    {
      sText = "UserLight" + toString (iCount);
      oSceneMgr->destroyLight (sText.c_str());
    }
    iNumUserLights = 0;
  }
  else
  if (cmpCommand ("Toggle Mobile Light"))
  {
    if (!bMobileLight)
    {
      oUserLight = oSceneMgr->createLight ("MobileLight");
      oUserLight->setType (Light::LT_POINT);
      oUserLight->setDiffuseColour (ColourValue::White);
      oUserLight->setSpecularColour (ColourValue::Black);
      oCameraRollNode->attachObject (oUserLight);
      bMobileLight = true;
    }
    else
    {
      oSceneMgr->destroyLight ("MobileLight");
      bMobileLight = false;
    }
  }
  else
  if (cmpCommand ("Ogre FPS"))
  {
    if (bStatsOn)
    {
      oOgreOverlay->hide();
      bStatsOn = false;
    }
    else
    {
      oOgreOverlay->show();
      bStatsOn = true;
    }
  }
  else
  if (cmpCommand ("FPS"))
  {
    if (bSimpleFPS)
    {
      OverlayManager::getSingleton().getOverlayElement ("Solar/SimpleFPS")->
        setCaption ("");
      bSimpleFPS = false;
    }
    else
      bSimpleFPS = true;
  }
  else
  if (cmpCommand ("Screenshot"))
  {
    setCommandText (NULL);
    showDebugText (NULL);
    sScreenShotFileName = "../screenshot_" + toString (++iScreenShotCount) +
      ".png";
  }
  else
  if (cmpCommand ("Reset View"))
    resetView();
  else
  if (cmpCommand ("Change Screen Mode"))
  {
      iReturnCode = sv_exit_screen_mode;
      bContinue = false;
  }
  else
  if (cmpCommand ("Time Forward"))
  {
    iSolarTimeOrigin = iSolarVisitorTime;
    iTimeWarp = 200;
    iUserTimeOrigin = getUserTime();
  }
  else
  if (cmpCommand ("Time Backward"))
  {
    iSolarTimeOrigin = iSolarVisitorTime;
    iTimeWarp = -200;
    iUserTimeOrigin = getUserTime();
  }
  else
  if (cmpCommand ("Time Stop"))
  {
    iSolarTimeOrigin = iSolarVisitorTime;
    iTimeWarp = 0;
    iUserTimeOrigin = getUserTime();
  }
  else
  if (cmpCommand ("Time Epoch"))
  {
    iSolarTimeOrigin = 0; ///// iSolarVisitorTime;
    iSolarVisitorTime = 0;
    iSolarVisitorTimeLast = !iSolarVisitorTime;
    iTimeWarp = 0;
    iUserTimeOrigin = getUserTime();
  }
}


void Listener::updateStats()
{
  OverlayElement* oe;

  if (!bStatsOn && !bSimpleFPS)
    return;

  const RenderTarget::FrameStats &stats = oRenderWindow->getStatistics();

  if (bStatsOn)
  {
    oe = OverlayManager::getSingleton().getOverlayElement ("Core/CurrFps");
    oe->setCaption ("Current FPS: " + toString (stats.lastFPS));

    oe = OverlayManager::getSingleton().getOverlayElement ("Core/AverageFps");
    oe->setCaption ("Average FPS: " + toString (stats.avgFPS));

    oe = OverlayManager::getSingleton().getOverlayElement ("Core/WorstFps");
    oe->setCaption ("Worst FPS: " + toString (stats.worstFPS) +
     + " (" + toString (stats.worstFrameTime) +"ms)");

    oe = OverlayManager::getSingleton().getOverlayElement ("Core/BestFps");
    oe->setCaption ("Best FPS: " + toString (stats.bestFPS) +
      " (" + toString (stats.bestFrameTime) +"ms)");

    oe = OverlayManager::getSingleton().getOverlayElement ("Core/NumTris");
    oe->setCaption ("Triangle Count: " + toString (stats.triangleCount));
  }
  
  if (bSimpleFPS)
  {
    oe = OverlayManager::getSingleton().getOverlayElement ("Solar/SimpleFPS");
    oe->setCaption (toString (stats.lastFPS));
  }
}


