/*

This source file is part of the Solar Visitor program, (c) 2007
Graeme Sheppard. Released under the GPLv2.

*/


#include "SolarSystem.h"


SolarSystem::SolarSystem (SceneManager *sm)
{
  oSceneManager = sm;
}


void SolarSystem::addPlanets()
{
  Entity *ent;
  SceneNode *node;

  ent = oSceneManager->createEntity ("Sun", "Sun1.mesh");
  node = oSceneManager->getRootSceneNode()->createChildSceneNode ("Sun1",
    Vector3 (0, 0, 0));
  node->attachObject (ent);

  addPlanet ("Mercury", "Mercury1.mesh", 1.0, "Sun", 5000.0, -28.0, -6.0,
             0.5, 0.0, 1.0, -10.0, 0.0, 8.0);
  addPlanet ("Venus", "Venus1.mesh", 1.0, "Sun", 10000.0, 31.0, 2.5,
             0.0, 0.0, -1.0, 0.0, 0.0, -8.0);
  addPlanet ("Earth", "Earth8.mesh", 1.0, "Sun", 15000.0, 36.525, 4.0,
             1.0, 0.0, -1.0, 0.0, 0.0, 8.0);
  addPlanet ("Mars",  "Mars2.mesh", 1.0, "Sun", 20000.0, -40.0, -5.0,
             -0.5, 0.0, -1.0, -10.0, 0.0, -10.0);
  addPlanet ("Jupiter", "Jupiter1.mesh", 1.0, "Sun", 25000.0, 60.0, 7.0,
             2.0, 0.0, -1.0, 0.0, 0.0, 9.0);
  addPlanet ("Saturn", "Saturn4.mesh", 1.0, "Sun", 30000.0, 45.0, 8.0,
             -1.5, 0.0, 0.7, 10.0, 0.0, 20.0);
  addPlanet ("Uranus", "Uranus1.mesh", 1.0, "Sun", 35000.0, -60.0, -10.0,
             1.2, 0.0, 0.0, 10.0, 0.0, 5.0);
  addPlanet ("Neptune", "Neptune1.mesh", 1.0, "Sun", 40000.0, 70.0, 9.0,
             -1.1, 0.0, 1.0, 0.0, 0.0, -9.0);
  addPlanet ("Pluto", "Pluto1.mesh", 1.0, "Sun", 45000.0, -90.0, 9.0,
             0.0, 0.0, 0.5, 8.0, 0.0, 8.0);

  // list moons last
  addPlanet ("Moon", "Moon3.mesh", 1.0, "Earth", 2250.0, 8.0, 0.0,
             5.0, 0.0, -10.0, 0.0, 0.0, 0.0);
}


void SolarSystem::addBody (Planet *oBody, Real rScale)
{
  SceneNode *oNode;
  Entity *oEntity;

  oPlanets.push_back (oBody);
  updatePosition (oBody, 0); // the time
  oEntity = oSceneManager->createEntity (oBody->sName, oBody->sMesh);
  // using identity position and no tilt
  if (!oEntity)
    throw Exception (1, "Cannot create entity - missing mesh?", "addBody()");
  oNode = oSceneManager->getRootSceneNode()->createChildSceneNode
    (oBody->sName);
  oNode->attachObject (oEntity);
  oNode->setScale (rScale, rScale, rScale);
}


void SolarSystem::addPlanet (const char *szName, const char *szMesh,
  Real rScale, const char *szFacing,
  Real rRadius, Real rYear, Real rSpin,
  Real rXorb, Real rYorb, Real rZorb, Real rXort, Real rYort, Real rZort)
{
  Planet *oPlanet = new Planet();
  
  oPlanet->sName = String (szName);
  oPlanet->sMesh = String (szMesh);
  oPlanet->sOrbiting = String (szFacing);
  oPlanet->rOrbitRadius = rRadius;
  oPlanet->rYear = rYear;
  oPlanet->rSpin = rSpin;
  oPlanet->rInitialAngle = 0.0;
  oPlanet->oOrbitSlant =
    Quaternion (Degree (rXorb), Vector3::UNIT_X) *
    Quaternion (Degree (rYorb), Vector3::UNIT_Y) *
    Quaternion (Degree (rZorb), Vector3::UNIT_Z);
  oPlanet->oSlant =
    Quaternion (Degree (rXort), Vector3::UNIT_X) *
    Quaternion (Degree (rYort), Vector3::UNIT_Y) *
    Quaternion (Degree (rZort), Vector3::UNIT_Z);
  oPlanet->oOrbitSlant.normalise();
  oPlanet->oSlant.normalise();
  addBody (oPlanet, rScale);
}


Planet* SolarSystem::findPlanet (const char *szName)
{
  Planet *oPlanet;
  unsigned int iCount;
  
  for (iCount = 0; iCount < oPlanets.size(); iCount++)
  {
    oPlanet = oPlanets [iCount];
    if (oPlanet->sName == szName)
      return oPlanet;
  }
  return NULL;
}


Radian circleFraction (Real rYear, sv64 iTime)
{
  Real rTemp;
  
  rTemp = iTime / (rYear * (1000.0 / Math::TWO_PI));
  rTemp = rTemp - int (rTemp);
  if (rTemp < 0.0)
    rTemp += 1.0;
  return Radian (rTemp * Math::TWO_PI);
}


void SolarSystem::updatePosition (Planet *oPlanet, sv64 iTime)
{
  Quaternion oQspin;
  Radian rOrbit, rSpin;
  Planet *oCoo; // centre of orbit
  Real rCooX, rCooY, rCooZ;
  Vector3 oVector;

  if (oPlanet->sOrbiting == "Sun")
    rCooX = rCooY = rCooZ = 0.0;
  else
  {
    oCoo = findPlanet (oPlanet->sOrbiting.c_str());
    if (oCoo == NULL) // should always be non-null
      return;
    rCooX = oCoo->rX;
    rCooY = oCoo->rY;
    rCooZ = oCoo->rZ;
  }

  rOrbit = circleFraction (oPlanet->rYear, iTime);
  oVector = Vector3 (Math::Cos (rOrbit), 0, Math::Sin (rOrbit));
  oVector *= oPlanet->rOrbitRadius;
  oVector = oPlanet->oOrbitSlant * oVector;
  oPlanet->rX = rCooX + oVector.x;
  oPlanet->rY = rCooY + oVector.y;
  oPlanet->rZ = rCooZ + oVector.z;

  rSpin = circleFraction (oPlanet->rYear, iTime);
  /// if (oPlanet->sOrbiting != "Sun")...
  oQspin = Quaternion (rSpin, Vector3::UNIT_Y);
  oPlanet->oOrientationNow = oPlanet->oSlant * oQspin * oBlenderAdjustAngle;
}

